@@ -12,7 +12,7 @@ import abc
 import math
 from typing import Literal
 
-from .utility import choose, factorial, round_sig_fig
+from .utility import choose, round_sig_fig
 
 
 class NonsenseError(Exception):
@@ -320,8 +320,10 @@ class BinomialDistribution(Distribution):
         :raises NonsenseError: If the number of successes is outside the valid range
         :raises NonsenseError: If the number of successes is not an integer
         """
-        return 0 if self.check_nonsense(successes, strict=strict) is not None else \
-            self._choose(successes) * (self._probability ** successes) * \
+        if self.check_nonsense(successes, strict=strict) is not None:
+            return 0
+
+        return self._choose(successes) * (self._probability ** successes) * \
             ((1 - self._probability) ** (self._number_of_trials - successes))
 
     def cdf(self, successes: int, *, strict: bool = True) -> float:
@@ -336,9 +338,25 @@ class BinomialDistribution(Distribution):
         :raises NonsenseError: If the number of successes is outside the valid range
         :raises NonsenseError: If the number of successes is not an integer
         """
+        if self.check_nonsense(successes, strict=strict) is not None:
+            return 0
+
+        if successes == self._number_of_trials:
+            return 1
+
         # mypy expects this sum to have ints for some reason, so we ignore it
-        return 0 if self.check_nonsense(successes, strict=strict) is not None else \
-            sum(self.pmf(x) for x in range(successes + 1))  # type: ignore[misc]
+        return sum(self.pmf(x) for x in range(successes + 1))  # type: ignore[misc]
+
+    def calculate(self, *, strict: bool = True) -> float:
+        """Check for nonsense in an edge case.
+
+        This method overrides :meth:`Distribution.calculate`. See that method for documentation.
+        """
+        if self.bounds.lower == (self._number_of_trials, False):
+            raise NonsenseError(f'Cannot have more successes (> {self._number_of_trials}) '
+                                f'than trials ({self._number_of_trials})')
+
+        return super().calculate(strict=strict)
 
 
 class PoissonDistribution(Distribution):
@@ -397,8 +415,16 @@ class PoissonDistribution(Distribution):
         :raises NonsenseError: If the number of occurrences is negative
         :raises NonsenseError: If the number of occurrences is not an integer
         """
-        return 0 if self.check_nonsense(number, strict=strict) else \
-            float((math.e ** -self._rate * self._rate ** number) / factorial(number))
+        if self.check_nonsense(number, strict=strict) is not None:
+            return 0
+
+        if number == 0:
+            return math.exp(-self._rate)
+
+        # This line is pure magic that I stole from the SciPy source code
+        # https://github.com/scipy/scipy/blob/main/scipy/stats/_discrete_distns.py#L854-L860
+        log_pmf = number * math.log(self._rate) - math.lgamma(number + 1) - self._rate
+        return math.exp(log_pmf)
 
     def cdf(self, number: int, *, strict: bool = True) -> float:
         """Return the probability that we get less than or equal to the given number of occurrences.
@@ -412,8 +438,10 @@ class PoissonDistribution(Distribution):
         :raises NonsenseError: If the number of occurrences is negative
         :raises NonsenseError: If the number of occurrences is not an integer
         """
-        return 0 if self.check_nonsense(number, strict=strict) is not None else \
-            sum(self.pmf(x) for x in range(number + 1))  # type: ignore[misc]
+        if self.check_nonsense(number, strict=strict) is not None:
+            return 0
+
+        return sum(self.pmf(x) for x in range(number + 1))  # type: ignore[misc]
 
 
 class ProbabilityCalculator:
@@ -449,6 +477,13 @@ class ProbabilityCalculator:
 
         :raises NonsenseError: If the bounds of the distribution are invalid
         """
-        probability = distribution.calculate(strict=True)
-        distribution.reset()
+        try:
+            probability = distribution.calculate(strict=True)
+
+        except NonsenseError as e:
+            raise e
+
+        finally:
+            distribution.reset()
+
         return probability
